#include <iostream> 
#include <string>
#include <random>
#include <conio.h>
#include <chrono>
#include <thread>

void wait_ms(unsigned int milliseconds)
{
    std::this_thread::sleep_for(std::chrono::milliseconds(milliseconds));
}

int get_random_integer(int n1, int n2)
{
    static std::random_device rd;
    static std::mt19937 gen(rd());
    std::uniform_int_distribution<> dis(n1, n2);

    return dis(gen);
}

struct Getter
{
    virtual char operator()() { return 0; };
};

struct KeyboardGetter : public Getter
{
    char operator ()() override
    {
        return getch();
    }
};

struct RandomCharacterGetter : public Getter
{
    char operator ()() override
    {
        wait_ms(100);
        static const std::string alphanum = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
        return alphanum[get_random_integer(0, alphanum.size() - 1)];
    }
};

struct Putter
{
    void operator()(char character)
    {
        std::cout << character;
    }
};

struct Copy
{
    // dlatego, e nie ufam wskanikom
    std::shared_ptr<Getter> getter_; 

    Copy(std::shared_ptr<Getter> getter)
        : getter_(getter)
    {
    }

    void operator()()
    {
        while (1)
        {
            auto character = (*getter_)();
            if (character == 'x') break;
            Putter()(character);
        }
    }
};

// oddzielam proces tworzenia obiektu od uycia poprzez 
// utworzenie fabryki obiektw
struct GetterFactory
{
    std::shared_ptr<Getter> operator()()
    {
        // tutaj jest dobre miejsce na dodanie logiki wybierajcej 
        // ktra klasa powinna by stworzona
        //return std::make_shared<KeyboardGetter>();
        return std::make_shared<RandomCharacterGetter>();
    }
};

int main(void)
{
    auto getter = GetterFactory()();

    (Copy(getter))();

    std::cout << std::endl;
    return 0;
}
